/*
* All questions regarding the software should be addressed to
* 
*       Prof. Charles A. Bouman
*       Purdue University
*       School of Electrical and Computer Engineering
*       1285 Electrical Engineering Building
*       West Lafayette, IN 47907-1285
*       USA
*       +1 765 494 0340
*       +1 765 494 3358 (fax)
*       email:  bouman@ecn.purdue.edu
*       http://www.ece.purdue.edu/~bouman
* 
* Copyright (c) 1995 The Board of Trustees of Purdue University.
*
* Permission to use, copy, modify, and distribute this software and its
* documentation for any purpose, without fee, and without written agreement is
* hereby granted, provided that the above copyright notice and the following
* two paragraphs appear in all copies of this software.
*
* IN NO EVENT SHALL PURDUE UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
* USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF PURDUE UNIVERSITY HAS
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
* PURDUE UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS,
* AND PURDUE UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
* UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

/* clustermessage.c */

#include <stdlib.h>                    /* exit */
#include <stdarg.h>                    /* va_list, va_start, va_end */
#include <stdio.h>                     /* fprintf, vfprintf */
#include "clustermessage.h"            /* clusterMessageInit, clusterInfo,    */
                                       /* clusterWarning, clusterError,       */
                                       /* INFO, INFO_END, messageVerboseLevel */           

/* global declarations */

int clusterMessageVerboseLevel = 0;    /* default is 0 (no messages) */


/* default so that clusterMessageInit does not need to be run */
static char * programName = "cluster";        /* default */


void clusterMessageInit( char * currentProgramName ) {
    programName = currentProgramName;
} /* clusterMessageInit */


void clusterMessageTerm( void ) {
    /* do nothing */
} /* clusterMessageTerm */


void clusterInfo( const char * format,... ) {
    va_list ap;
    va_start(ap, format);
    
    /* fprintf (stdout, "Info: %s: ", programName); */
    vfprintf(stdout, format, ap);
    fprintf (stdout, "\n");

    va_end(ap);
} /* defaultInfo */


void clusterWarning( const char * format,... ) {
    va_list ap;
    va_start(ap, format);
    
    fprintf (stdout, "Warning: %s: ", programName);
    vfprintf(stdout, format, ap);
    fprintf (stdout, "\n");

    va_end(ap);
} /* defaultWarning */


void clusterError( int errorNumber, const char * format,... ) {
    va_list ap;
    va_start(ap, format);
    
    fprintf (stdout, "Error %d: %s: ", errorNumber, programName);
    vfprintf(stdout, format, ap);
    fprintf (stdout, "\n");

    va_end(ap);

    exit( errorNumber );
} /* defaultError */
